/*
    Anagrammimylly: an anagram generator
    Copyright (C) 1998-2000 Pauli Virtanen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/*
 * Interface header to the anagrammimylly library
 * (actually it should be merged into the library itself...)
 */
#ifndef ANAGRAM_H
#define ANAGRAM_H

#include "wordlist.h"
#include <ctype.h>

#ifdef __cplusplus
extern "C" {
#endif

/*
  Simple user interface.
*/
        char* simple_anagram(char *dictfile, char *word, int index);
        void simple_anagram_free(char *data);


/*
  Functions return always nonzero on success and zero on error
  anagram_wl_fix returns the count of removed words
*/
        typedef success_t outfunc_t(char *, void *);
        typedef struct anagram_data_t {
                char            wl_loaded;
                wordlist_t      wl;
        
                void            *user_data;
                outfunc_t	*output_word;
                outfunc_t       *output_info;
        } anagram_data_t;

        success_t anagram_init(anagram_data_t *data);

        success_t anagram_set_output_functions(outfunc_t *outlist,
                                               outfunc_t *outinfo,
                                               void *user_data,
                                               anagram_data_t *data);

        success_t anagram_wl_load(char *wl, anagram_data_t *data);
        success_t anagram_wl_save(char *wl, anagram_data_t *data);
    
        unsigned long anagram_wl_get_count(anagram_data_t *data);

        success_t anagram_wl_add_word(char *word, anagram_data_t *data);
        success_t anagram_wl_remove_word(char *word, anagram_data_t *data);
        success_t anagram_wl_add_file(char *file, anagram_data_t *data);
        unsigned long anagram_wl_fix(anagram_data_t *data, char dup,
                                     char low, char one, char two,
                                     char sort, char remlong);

        success_t anagram_find_word(char *word, char *must_have,
                                    anagram_data_t *data);

        success_t anagram_uninit(anagram_data_t *data);

	/* Simple 8-bit isalpha */
	#define my_isalpha(c) (!isspace(c))
	
#ifdef __cplusplus
}
#endif
#endif
