/*
    Anagrammimylly: an anagram generator
    Copyright (C) 1998-2000 Pauli Virtanen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/*
 * A simple wordlist, STLish implementation
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "wordlist.h"

#define RemoveEndChar(str, chr) {if (strlen(str) > 0 && str[strlen(str)-1] == chr) str[strlen(str)-1] = '\0';}

#define RemoveEndLine(str) { while (str[strlen(str)-1] == '\n' || str[strlen(str)-1]  == '\r') RemoveEndChar(str, str[strlen(str)-1]); }


success_t wl_save(wordlist_t *wordlist, char *file)
{
	FILE *wordlistfile;

        if ((wordlistfile = fopen(file, "wt")) != NULL)
	{
                word_t *w;
                for (w = wordlist->begin; w != wordlist->end; ++w)
			fprintf(wordlistfile, "%s\n", w->word);
		fclose(wordlistfile);
		return success;
	}
	return failure;
}
success_t wl_load(wordlist_t *wordlist, char *file)
{
	FILE *wordlistfile = fopen(file, "rt");
	char buffer[100];

	if (wordlistfile == NULL) return failure;

        wordlist->begin = NULL;
        wordlist->end = NULL;
        wordlist->alloc_end = NULL;

	while (fgets(buffer, 100, wordlistfile))
	{
		RemoveEndLine(buffer);
		if (buffer[0] != '\0')
                        if (wl_add_word(wordlist, buffer) != success)
                                return failure;
	}
	fclose(wordlistfile);
	return success;
}
void wl_unload(wordlist_t *wordlist)
{
        word_t *w;
        for (w = wordlist->begin; w != wordlist->end; ++w)
                free(w->word);
        if (wordlist->begin) free(wordlist->begin);
        
        wordlist->begin = NULL;
        wordlist->end = NULL;
        wordlist->alloc_end = NULL;
}

success_t wl_add_word(wordlist_t *wordlist, char *text)
{
        if (wordlist->end < wordlist->alloc_end)
        {
                if (text == NULL)
                        wordlist->end->end = wordlist->end->word = NULL;
                else
                {
                        int len = (strlen(text) + 1) * sizeof(char);
                        wordlist->end->word = (char*)malloc(len);
                        if (wordlist->end->word == NULL)
                                return failure;
                        memcpy(wordlist->end->word, text, len);
                        wordlist->end->end = wordlist->end->word + len - 1;
                }
                ++wordlist->end;
                return success;
        }
        else
        {
                word_t *n;
                int nl;
                int ul = wordlist->end - wordlist->begin;
                if (wordlist->begin == NULL)
                {
                        nl = 10;
                        n = (word_t*)malloc(nl*sizeof(word_t));
                }
                else
                {
                        nl = (wordlist->alloc_end - wordlist->begin)*2 + 10;
                        n = (word_t*)realloc(wordlist->begin,
                                             nl*sizeof(word_t));
                }
                if (n == NULL)
                        return failure;
                wordlist->begin = n;
                wordlist->alloc_end = n + nl;
                wordlist->end = n + ul;

                return wl_add_word(wordlist, text);
        }
}

unsigned long wl_get_count(wordlist_t *wordlist)
{
        return wordlist->end - wordlist->begin;
}

success_t wl_remove(wordlist_t *wordlist, word_t *w)
{
        /* bounds check */
        if (w < wordlist->begin || w >= wordlist->end)
                return failure;
        memcpy(w, w + 1, wordlist->end - w);
        --wordlist->end;
        return success;
}

word_t *wl_find_word(wordlist_t *wordlist, char *word)
{
        word_t *w;
        for (w=wordlist->begin; w != wordlist->end; ++w)
		if (strcmp(word, w->word) == 0) return w;
        return wordlist->end;
}
