/*
    Anagrammimylly: an anagram generator
    Copyright (C) 1998-2000 Pauli Virtanen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/*
 * lex_fix: performing certain "fix" operations on simple wordlist
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "anagram_internal.h"
#include "anagram.h"
#include "wordlist.h"


#define isclauseatstart(str1, str2) (strstr(str1, str2) == str1)


unsigned long wl_fix(wordlist_t *wordlist, outfunc_t *outinfo,
                     void *user_data,
                     char dup, char low, char one, char two,
                     char sort, char remlong)
{
        wordlist_t new_list;
        
	word_t *w;
	unsigned long rem_cnt = 0;

        new_list.begin = NULL;
        new_list.end = NULL;
        new_list.alloc_end = NULL;
        
        for (w = wordlist->begin; w != wordlist->end; ++w)
	{
		if (!w->word)
                        continue;
                
                if (outinfo != NULL && (w - wordlist->begin) % 100 == 0 &&
                    (*outinfo)(w->word, user_data) != success)
                        return rem_cnt; // Inform the user, and allow termination.

                // Remove oneletter words
                if (one && (w->end - w->word) <= 1)
                {
                        ++rem_cnt;
                        continue;
                }
                // Remove twoletter words
                if (two && (w->end - w->word) == 2)
                {
                        ++rem_cnt;
                        continue;
                }

                // remove too long words
                if (remlong && (w->end - w->word) > 10)
                {
                        ++rem_cnt;
                        continue;
                }

                // remove duplicate words
                if (dup && w != wordlist->begin)
                {
                        word_t *v;
                        for (v = wordlist->begin; v != w; ++v)
                                if (strcmp(w->word, v->word) == 0)
                                        break;
                        if (v != w)
                        {
                                ++rem_cnt;
                                continue;
                        }
                }

                // Convert to lowercase
                if (low) str_tolower(w->word    );

                // Add to the new dictionary
                wl_add_word(&new_list, w->word);
        }

        // Copy the new list to the old
        wl_unload(wordlist);
        memcpy(wordlist, &new_list, sizeof(wordlist_t));

        return rem_cnt;
}
